@@ -1,7 +1,8 @@
+{-# OPTIONS_GHC -Wno-unused-do-bind #-}
 {-# LANGUAGE CPP #-}
 {-|
 Module      : Main
-Copyright   : © 2021 Albert Krewinkel
+Copyright   : © 2021-2022 Albert Krewinkel
 License     : MIT
 Maintainer  : Albert Krewinkel <tarleb+hslua@zeitkraut.de>
 Stability   : beta
@@ -24,6 +25,8 @@ import Foreign.Storable as Storable
 import Lua
 import Test.Tasty (TestTree, defaultMain, testGroup)
 import Test.Tasty.HUnit (Assertion, HasCallStack, assertBool, testCase, (@=?))
+import qualified Lua.ErsatzTests
+import qualified Lua.PrimaryTests
 import qualified Lua.UnsafeTests
 
 -- | Runs tests.
@@ -112,6 +115,11 @@ tests = testGroup "lua"
         LUA_TNUMBER `shouldBeResultOf` \l -> do
           lua_pushinteger l 0
           lua_type l top
+
+    , "show Integer" =: ("23" @=? show (23 :: Lua.Integer))
+    , "read Integer" =: ((23 :: Lua.Integer) @=? read "23")
+    , "show Number" =: ("23.5" @=? show (23.5 :: Lua.Number))
+    , "read Number" =: ((23.5 :: Lua.Number) @=? read "23.5")
     ]
 
   , testGroup "nil"
@@ -243,26 +251,6 @@ tests = testGroup "lua"
           lua_pcall l (NumArgs 1) (NumResults 1) 0
     ]
 
-  , testGroup "garbage-collection"
-    [ "stop, restart GC"  =: do
-        counts <- withNewState $ \l -> do
-          lua_createtable l 0 0
-          _  <- lua_gc l LUA_GCSTOP 0
-          lua_pop l 1
-          kb1 <- lua_gc l LUA_GCCOUNT 0
-          b1  <- lua_gc l LUA_GCCOUNTB 0
-          _   <- lua_gc l LUA_GCCOLLECT 0
-          kb2 <- lua_gc l LUA_GCCOUNT 0
-          b2  <- lua_gc l LUA_GCCOUNTB 0
-          return (b1 + 1024 * kb1, b2 + 1024 * kb2)
-        assertBool "first count should be larger" (uncurry (>) counts)
-    , "count memory" =: do
-        count <- withNewState $ \l -> do
-          lua_gc l LUA_GCCOUNT 0
-        assertBool "memory consumption not between 0 and 10 kB"
-                   (count > 0 && count < 10)
-    ]
-
   , testGroup "ersatz functions"
     [ testGroup "globals"
       [ "get global from base library" =:
@@ -389,7 +377,9 @@ tests = testGroup "lua"
 #endif
     ]
 
+  , Lua.PrimaryTests.tests
   , Lua.UnsafeTests.tests
+  , Lua.ErsatzTests.tests
   ]
 
 infix  3 =:
